/*
	animated_monsters.cpp
	A dynRPG plugin to animate your monsters with the DBS!
	Feel free to use or modify this as you see fit.
	Your safety is not guaranteed.

	--dragonheartman
	6 JUN 2012

	Thanks, Cherry for teaching me the config and file check
	codes. This really is awesome code.
	--Thomas Hooker (bulmabriefs144)


*/

#define AUTO_DLLMAIN
#define NUM_POSES 7
#define MAX_MONSTERS 8

#include <DynRPG/DynRPG.h>
#include <sstream>
#include <fstream>

RPG::Image* monsterPoses[MAX_MONSTERS][NUM_POSES];		/* table containing all images created */
RPG::Image* oldPose;									/* rm2k3's old monster pose, saved off */
boolean resetTable = false;								/* reset table flag */
int frameTimer = 0;										/* global frame updater */
int hurtTimer[MAX_MONSTERS];							/* a timer for how many frames we ought to show the hurt pose */
int oldHP[MAX_MONSTERS];								/* old hp vals to check for damaged enemy */
int poseIndex[MAX_MONSTERS];							/* which monster pose do we show? */
int animationCycle[NUM_POSES] = {1, 2, 3, 2, 4, 5, 6};	/* index for which animation file to display */
int animationType; 										/* which animation type? 0=idle, 1=hurt, 2=dead */
/* 1-3 idle, 4 dead, 5-6 hurt */

// We store the configuration here
std::map<std::string, std::string> configuration;

// We store the images of the conditions here
std::map<int, RPG::Image *> images;


// Checks if a file exists

bool doesFileExist(std::string filename) {

	std::ifstream ifile(filename.c_str()); // Try to open file for reading

	return (bool)ifile; // Casted to bool here, so true if ifile != null, i.e. it exists

}



// Checks if a monster is animated, by checking if there is a 1.png file for it

bool isBattlerAnimated(RPG::Battler *battler) {

	// Build full path string (e.g. Monster\Slime\1.png)

	std::stringstream fileName;

	fileName << "Monster\\" << battler->getName() << "\\1.png";

	// Check if the file exists and return result

	return doesFileExist(fileName.str());

}


bool onStartup(char *pluginName) {
               // We load the configuration from the DynRPG.ini file here
               configuration = RPG::loadConfiguration(pluginName);
               //not sure if it goes inside here or outside, or how to define it


               return true; // Don't forget to return true so that the start of the game will continue!
}


// destroy ALL the images!!
void initializeBattle()
{
    // update our initial pose indexes (0-3)
    for (int i=0; i<MAX_MONSTERS; i++)
    {
        poseIndex[i]=i%4;
        hurtTimer[i]=0;
        oldHP[i]=0;

        // destroy any remaining images -- no leaks pls
        for (int j=0; j<NUM_POSES; j++)
        {
			if(monsterPoses[i][j])
			{
				RPG::Image::destroy(monsterPoses[i][j]);
			}
        }
    }
}

// called every time a batter is drawn
bool onDrawBattler(RPG::Battler *battler, bool isMonster, int id)
{


    if(isMonster && isBattlerAnimated(battler))
    {

        // if the current HP is less than old HP, we need to set the appropriate hurt timer
        // else: reset oldhp in the case where the enemy is healed
        if(battler->hp < oldHP[id])
        {
            hurtTimer[id] = 30;
            oldHP[id] = battler->hp;
        }
        else if(battler->hp > oldHP[id])
        {
            oldHP[id] = battler->hp;
        }

        // if the monster has hurt poses to display, we need to hard set the frame index to 5 (HURT)
        if(hurtTimer[id] > 0)
        {
            poseIndex[id] = 5 + (frameTimer-1)/5;
            hurtTimer[id]--;
        }

        // if the monster dies, we want to hard set the frame index to 4 (DEAD)
        if(battler->hp <= 0)
        {
            poseIndex[id]=4;
        }

        if(!monsterPoses[id][animationCycle[poseIndex[id]]])
        {
            // create the new blank image
			monsterPoses[id][animationCycle[poseIndex[id]]] = RPG::Image::create();
			monsterPoses[id][animationCycle[poseIndex[id]]]->useMaskColor = true;

			// load the image file into our image we created
			std::stringstream fileName;
			//The issue is here. We need a branch so it normally loads without creating an animation cycle
			//then if a folder exists it needs to check the animation cycle.

			fileName << "Monster\\" << battler->getName() << "\\" << animationCycle[poseIndex[id]] << ".png";
        	monsterPoses[id][animationCycle[poseIndex[id]]]->loadFromFile(fileName.str(), true);


        }
		// we do this (UGLY) cast to update the monster's image to the image in the loader table
        oldPose = static_cast<RPG::Monster*>(battler)->image;
		static_cast<RPG::Monster*>(battler)->image = monsterPoses[id][animationCycle[poseIndex[id]]];

    }
    return true;
}

// after the battler is drawn, we need to set the monster's default image to point back to the saved off image
bool onBattlerDrawn(RPG::Battler *battler, bool isMonster, int id)
{
    //possibly add the keyname section here too, not sure.

    //s for string length
    //probably change this to isAnimated too?
    if(isMonster && isBattlerAnimated(battler))
    {
        static_cast<RPG::Monster*>(battler)->image = oldPose;
    }
    return true;
}


// called every frame refresh
void onFrame(RPG::Scene scene)
{

    // for battle scenes only, we update a frame timer
    if(RPG::system->scene == RPG::SCENE_BATTLE)
    {
        frameTimer++;

        // every ten frames, update the index to be displayed in our monster's image
        if(frameTimer > 10)
        {
            frameTimer=0;
            for(int i=0; i<MAX_MONSTERS; i++)
            {
                if(hurtTimer[i]==0)
                {
                    poseIndex[i] = (poseIndex[i]+1)%4;
                }
            }
        }

        // set a flag notifying that we need to reset the loader table after combat
        resetTable = true;

    }
    else if (resetTable)
    {

        // reset the loader table post-battle
        initializeBattle();
        resetTable = false;

    }
}
